from django.conf import settings
from django.urls import path
from django.views.generic import TemplateView

from jwt_allauth.constants import EMAIL_VERIFIED_REDIRECT, PASSWORD_SET_REDIRECT
from jwt_allauth.registration.email_verification.views import VerifyEmailView
from jwt_allauth.registration.views import RegisterView, UserRegisterView
from jwt_allauth.password_reset.views import SetPasswordView, DefaultSetPasswordView

urlpatterns = []


if getattr(settings, 'JWT_ALLAUTH_ADMIN_MANAGED_REGISTRATION', False):
    urlpatterns.extend([
        path('user-register/', UserRegisterView.as_view(), name='rest_user_register'),
        path('set-password/', SetPasswordView.as_view(), name='rest_set_password'),
        path('verification/<str:key>/', VerifyEmailView.as_view(), name='account_confirm_email'),
    ])

    # Only register the built-in HTML UI if no custom PASSWORD_SET_REDIRECT is configured
    if getattr(settings, PASSWORD_SET_REDIRECT, None) is None:
        urlpatterns.append(
            path(
                'set-password/default/',
                DefaultSetPasswordView.as_view(),
                name='jwt_allauth_default_set_password',
            )
        )

else:
    urlpatterns.append(path('', RegisterView.as_view(), name='rest_register'))

    if getattr(settings, 'EMAIL_VERIFICATION', False):
        urlpatterns.extend([
            path('verification/<str:key>/', VerifyEmailView.as_view(), name='account_confirm_email'),

            # This url is used by django-allauth and empty TemplateView is
            # defined just to allow reverse() call inside app, for example when email
            # with verification link is being sent, then it's required to render email
            # content.

            # account_confirm_email - You should override this view to handle it in
            # your API client somehow and then, send post to /verify-email/ endpoint
            # with proper key.
            # If you don't want to use API on that step, then just use ConfirmEmailView
            # view from:
            # django-allauth https://github.com/pennersr/django-allauth/blob/master/allauth/account/views.py
            path('account_email_verification_sent/', TemplateView.as_view(), name='account_email_verification_sent'),
        ])

        if getattr(settings, EMAIL_VERIFIED_REDIRECT, None) is None:
            urlpatterns.append(
                path('verified/', TemplateView.as_view(
                    template_name='email/verified.html'), name='jwt_allauth_email_verified'),
            )



from django.conf import settings
from rest_framework import status
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework_simplejwt.exceptions import TokenError, InvalidToken
from rest_framework_simplejwt.views import TokenObtainPairView
from rest_framework.throttling import AnonRateThrottle

from jwt_allauth.app_settings import LoginSerializer
from jwt_allauth.utils import get_user_agent, sensitive_post_parameters_m, build_token_response
from jwt_allauth.constants import REFRESH_TOKEN_COOKIE


class LoginView(TokenObtainPairView):
    serializer_class = LoginSerializer
    throttle_classes = [AnonRateThrottle]

    @sensitive_post_parameters_m
    def dispatch(self, *args, **kwargs):
        return super(LoginView, self).dispatch(*args, **kwargs)

    @get_user_agent
    def post(self, request: Request, *args, **kwargs) -> Response:
        # Authenticate and generate the tokens
        serializer = self.get_serializer(data=request.data)
        try:
            serializer.is_valid(raise_exception=True)
        except TokenError as e:
            raise InvalidToken(e.args[0])

        # MFA setup required branch (REQUIRED mode without MFA configured)
        if serializer.validated_data.get('mfa_setup_required'):
            return Response(
                {
                    "mfa_setup_required": True,
                    "setup_challenge_id": serializer.validated_data.get("setup_challenge_id"),
                },
                status=status.HTTP_200_OK,
            )

        # MFA verification required branch (user has MFA enabled)
        if serializer.validated_data.get('mfa_required'):
            return Response(
                {
                    "mfa_required": True,
                    "challenge_id": serializer.validated_data.get("challenge_id"),
                },
                status=status.HTTP_200_OK,
            )

        return build_token_response(
            refresh_token=serializer.validated_data['refresh'],
            access_token=serializer.validated_data['access']
        )



"""
Custom MFA Adapter for JWT All-Auth.

This module provides a customized MFA adapter that inherits from allauth's
DefaultMFAAdapter, allowing override of specific MFA behavior such as the
TOTP issuer configuration.
"""

from django.conf import settings

from allauth.mfa.adapter import DefaultMFAAdapter


class JWTAllAuthMFAAdapter(DefaultMFAAdapter):
    """
    Custom MFA adapter extending allauth's DefaultMFAAdapter with JWT-specific MFA handling.

    This adapter allows customization of MFA behavior for JWT authentication scenarios,
    specifically enabling the use of a custom TOTP issuer name configured via
    JWT_ALLAUTH_TOTP_ISSUER setting instead of the default allauth TOTP_ISSUER.

    Key Features:

        - Custom TOTP issuer configuration via JWT_ALLAUTH_TOTP_ISSUER setting
        - Fallback to site name if no custom issuer is configured
        - Full compatibility with allauth MFA functionality
        - Seamless integration with JWT authentication workflows

    Configuration:

        The adapter is automatically configured when jwt_allauth is installed in INSTALLED_APPS.

        To customize the TOTP issuer, set the following in your Django settings:

            JWT_ALLAUTH_TOTP_ISSUER = "Your App Name"

        Priority order for TOTP issuer:

            1. JWT_ALLAUTH_TOTP_ISSUER (if set)
            2. 'JWT-Allauth' (default if setting is not provided)
            3. Current site name (if JWT_ALLAUTH_TOTP_ISSUER is explicitly set to None or empty)

    Example:

        settings.py::

            INSTALLED_APPS = [
                ...
                'allauth',
                'allauth.account',
                'allauth.mfa',
                'jwt_allauth',
                ...
            ]

            # Optional: customize the TOTP issuer name
            JWT_ALLAUTH_TOTP_ISSUER = "My App"
    """

    def get_totp_issuer(self) -> str:
        """
        Return the TOTP issuer name that will be contained in the TOTP QR code.

        This method overrides the parent implementation to check for JWT_ALLAUTH_TOTP_ISSUER
        in Django settings with a default fallback.

        Priority order:

            1. JWT_ALLAUTH_TOTP_ISSUER (if provided in settings)
            2. 'JWT-Allauth' (default value)
            3. Current site name (only if JWT_ALLAUTH_TOTP_ISSUER is explicitly empty/None)

        Returns:
            str: The TOTP issuer name to be used in QR code generation.
                 Typically the application or organization name.

        Example:

            With default configuration (no JWT_ALLAUTH_TOTP_ISSUER set)::

                >>> adapter = JWTAllAuthMFAAdapter()
                >>> issuer = adapter.get_totp_issuer()
                >>> print(issuer)
                'JWT-Allauth'

            With custom JWT_ALLAUTH_TOTP_ISSUER::

                # settings.py: JWT_ALLAUTH_TOTP_ISSUER = "My App"
                >>> adapter = JWTAllAuthMFAAdapter()
                >>> issuer = adapter.get_totp_issuer()
                >>> print(issuer)
                'My App'
        """
        # Check for JWT_ALLAUTH_TOTP_ISSUER with 'JWT-Allauth' as default
        issuer = getattr(settings, "JWT_ALLAUTH_TOTP_ISSUER", 'JWT-Allauth')
        if not issuer:
            issuer = self._get_site_name()
        return issuer



#!/usr/bin/env python3
"""
JWT Allauth CLI tool for creating and managing projects
"""
import sys
import os
import argparse
import subprocess
import re


def main():
    """
    Main entry point for the JWT Allauth CLI
    """
    parser = argparse.ArgumentParser(
        description='JWT Allauth command line tool',
        usage='jwt-allauth <command> [options]'
    )
    parser.add_argument('command', help='Command to run (e.g. startproject)')

    # Parse just the command argument first
    args, remaining_args = parser.parse_known_args()

    if args.command == 'startproject':
        # Handle startproject command
        project_parser = argparse.ArgumentParser(
            description='Create a new Django project with JWT Allauth pre-configured',
            usage='jwt-allauth startproject <name> [directory] [options]'
        )
        project_parser.add_argument('name', help='Name of the project')
        project_parser.add_argument('directory', nargs='?', help='Optional directory to create the project in')
        project_parser.add_argument('--email', default='False', help='Email configuration (True/False)')
        project_parser.add_argument('--template', help='Template directory to use as a base')

        project_args = project_parser.parse_args(remaining_args)

        # Get project arguments
        project_name = project_args.name
        target_dir = project_args.directory or project_name
        email_config = project_args.email.lower() == 'true'
        template = project_args.template

        try:
            # Build command for running django-admin startproject
            cmd = ["django-admin", "startproject", project_name]

            # Add directory if specified
            if project_args.directory:
                cmd.append(project_args.directory)

            # Add template if specified
            if template:
                cmd.extend(["--template", template])

            # Run django-admin startproject
            print(f"Creating Django project '{project_name}'...")
            result = subprocess.run(cmd, capture_output=True, text=True)

            if result.returncode != 0:
                print(f"❌ Error creating project: {result.stderr}")
                return 1

            print(f"✅ Created Django project '{project_name}'")

            # Step 2: Now modify the created project for JWT Allauth
            # Path to settings file
            settings_path = os.path.join(target_dir, project_name, 'settings.py')

            # Modify settings.py to include JWT-allauth configuration
            _modify_settings(settings_path, email_config)
            print("✅ Added JWT Allauth configuration to settings.py")

            # Add urls.py configuration
            urls_path = os.path.join(target_dir, project_name, 'urls.py')
            _modify_urls(urls_path)
            print("✅ Added JWT Allauth URLs to urls.py")

            # Create templates directory if needed
            if email_config:
                templates_dir = os.path.join(target_dir, 'templates')
                os.makedirs(templates_dir, exist_ok=True)
                print("✅ Created templates directory")

            # Final instructions
            print("\n✅ JWT Allauth project successfully created!")
            print("📋 Next steps:")
            print(f"   1. cd {target_dir}")
            print(f"   2. python manage.py makemigrations")
            print(f"   3. python manage.py migrate")
            print(f"   4. python manage.py runserver")

            if email_config:
                print("\n⚠️ Email configuration is enabled. Please update your email settings in settings.py")

        except Exception as e:
            import traceback
            traceback.print_exc()
            print(f"❌ Unexpected error: {str(e)}")
            return 1

    elif args.command == 'help':
        parser.print_help()
        print("\nAvailable commands:")
        print("  startproject  - Create a new Django project with JWT Allauth pre-configured")

    else:
        print(f"❓ Unknown command: {args.command}")
        parser.print_help()
        return 1

    return 0

def _modify_settings(settings_path, email_config):
    """Modify Django settings.py to include JWT Allauth configuration"""
    with open(settings_path, 'r') as f:
        settings_content = f.read()

    # If email configuration will be added, ensure that the settings file
    # has access to the `os` and `secrets` modules so that generated email
    # settings can read credentials from environment variables or generate
    # dummy defaults without hard-coding real secrets in clear text.
    imports = []
    if email_config and "import os" not in settings_content:
        imports.append("import os")
    if email_config and "import secrets" not in settings_content:
        imports.append("import secrets")
    if imports:
        settings_content = "\n".join(imports) + "\n" + settings_content

    # Detect whether MFA support is available in the current environment.
    # We only configure MFA-related apps and settings when the optional
    # django-allauth[mfa] dependency is actually installed, to avoid
    # breaking generated projects.
    mfa_installed = False
    try:
        import allauth.mfa  # type: ignore  # noqa
        import allauth.mfa.totp  # type: ignore  # noqa
        import allauth.mfa.recovery_codes  # type: ignore  # noqa
        import jwt_allauth.mfa  # type: ignore  # noqa
    except Exception:  # pragma: no cover - purely defensive
        mfa_installed = False
    else:
        mfa_installed = True

    # Find INSTALLED_APPS content
    pattern = r"(INSTALLED_APPS\s*=\s*\[)(.*?)(,*\n*])"
    apps_lines = [
        "    'jwt_allauth',",
        "    'rest_framework',",
        "    'rest_framework.authtoken',",
        "    'allauth',",
        "    'allauth.account',",
        "    'allauth.socialaccount',",
    ]
    if mfa_installed:
        # Only add MFA-related apps when the optional allauth.mfa stack
        # is installed in the environment.
        apps_lines.append("    'allauth.mfa',")
    jwt_apps = "\n".join(apps_lines)

    # Replace keeping original Django apps and adding new ones before closing bracket
    settings_content = re.sub(pattern, fr'\1\2,\n{jwt_apps}\n]', settings_content, flags=re.DOTALL)

    # Add middleware
    pattern = r"(MIDDLEWARE\s*=\s*\[)(.*?)(,*\n*\])"
    allauth_middleware = "    'allauth.account.middleware.AccountMiddleware',"
    # Replace keeping original middleware and adding new one before closing bracket
    settings_content = re.sub(pattern, fr'\1\2,\n{allauth_middleware}\n]', settings_content, flags=re.DOTALL)

    # Add authentication backends
    auth_backends = """
# JWT Allauth user model
AUTH_USER_MODEL = 'jwt_allauth.JAUser'

# JWT Allauth adapter
ACCOUNT_ADAPTER = 'jwt_allauth.adapter.JWTAllAuthAdapter'

# Login configuration
ACCOUNT_LOGIN_METHODS = {'email'}
ACCOUNT_SIGNUP_FIELDS = ['email*', 'password1*', 'password2*']

# Authentication backends
AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend"
)

# Django Rest Framework settings
REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_simplejwt.authentication.JWTStatelessUserAuthentication',
    )
}

from datetime import timedelta

# JWT settings
JWT_ACCESS_TOKEN_LIFETIME = timedelta(minutes=30)
JWT_REFRESH_TOKEN_LIFETIME = timedelta(days=90)
"""
    settings_content += auth_backends

    # Only add MFA-related settings when the optional MFA dependencies
    # are installed. This keeps generated projects working even if
    # django-allauth[mfa] is not present.
    if mfa_installed:
        mfa_settings = """

# MFA configuration
JWT_ALLAUTH_MFA_TOTP_MODE = 'optional'
JWT_ALLAUTH_TOTP_ISSUER = None

# Cache configuration (required for MFA)
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake',
    }
}
"""
        settings_content += mfa_settings

    # Add email configuration if requested
    if email_config:
        email_settings = """
# Email configuration
EMAIL_VERIFICATION = True
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
EMAIL_HOST = os.environ.get('EMAIL_HOST', 'smtp.example.com')  # Configure or override via environment
EMAIL_PORT = int(os.environ.get('EMAIL_PORT', 587))
EMAIL_USE_TLS = os.environ.get('EMAIL_USE_TLS', 'True') == 'True'
EMAIL_HOST_USER = os.environ.get('EMAIL_HOST_USER') or f'dummy-user-{secrets.token_hex(4)}'
EMAIL_HOST_PASSWORD = os.environ.get('EMAIL_HOST_PASSWORD') or secrets.token_urlsafe(16)
DEFAULT_FROM_EMAIL = os.environ.get('DEFAULT_FROM_EMAIL', 'your-email@example.com')

# JWT Allauth settings
EMAIL_VERIFIED_REDIRECT = None  # URL to redirect after email verification
PASSWORD_RESET_REDIRECT = None  # URL for password reset form
"""
        settings_content += email_settings

    with open(settings_path, 'w') as f:
        f.write(settings_content)

def _modify_urls(urls_path):
    """Modify Django urls.py to include JWT Allauth URLs"""
    with open(urls_path, 'r') as f:
        urls_content = f.read()

    no_comments_urls_content = re.sub(r'""".*?"""', '', urls_content, flags=re.DOTALL)

    # Add import for include if needed
    if "from django.urls import path" in no_comments_urls_content:
        if "include" not in no_comments_urls_content:
            urls_content = urls_content.replace(
                "from django.urls import path",
                "from django.urls import path, include"
            )
    elif "include" not in no_comments_urls_content:
        urls_content = "from django.urls import include\n" + urls_content

    # Add JWT-allauth URLs
    urls_pattern = r"(urlpatterns\s*=\s*\[)(.*?)(,*\n*\])"
    jwt_urls = "    path('jwt-allauth/', include('jwt_allauth.urls')),"
    urls_content = re.sub(urls_pattern, fr'\1\2,\n{jwt_urls}\n]', urls_content, flags=re.DOTALL)

    with open(urls_path, 'w') as f:
        f.write(urls_content)

if __name__ == '__main__':
    sys.exit(main())



from __future__ import annotations

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.cache import cache
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.views import APIView

from jwt_allauth.constants import (
    MFA_TOKEN_MAX_AGE_SECONDS,
    MFA_TOTP_DISABLED,
    MFA_TOTP_REQUIRED,
)

from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.utils import build_token_response, load_user
from .serializers import (
    MFAActivateSerializer,
    MFAVerifySerializer,
    MFAVerifyRecoverySerializer,
    MFADeactivateSerializer,
    AuthenticatorSerializer,
)
from jwt_allauth.mfa.permissions import IsAuthenticatedOrHasMFASetupChallenge


def get_mfa_totp_mode() -> str:
    """
    Return the current MFA TOTP mode from settings.

    This must be evaluated at call time (not import time) so that
    Django's `override_settings` used in tests – and any runtime changes
    – are respected.
    """
    return getattr(settings, "JWT_ALLAUTH_MFA_TOTP_MODE", MFA_TOTP_DISABLED)

try:
    from allauth.mfa.models import Authenticator
    from allauth.mfa.totp.internal.auth import generate_totp_secret, TOTP
    from allauth.mfa.recovery_codes.internal.auth import RecoveryCodes
    from allauth.mfa.adapter import get_adapter
except Exception:  # pragma: no cover - optional dependency guard
    Authenticator = None  # type: ignore
    RecoveryCodes = None  # type: ignore
    generate_totp_secret = None  # type: ignore
    TOTP = None  # type: ignore
    get_adapter = None  # type: ignore

    if get_mfa_totp_mode() != MFA_TOTP_DISABLED:
        raise Exception(
            "MFA TOTP is not available. Please ensure 'django-jwt-allauth[mfa]' "
            "is installed and 'allauth.mfa' is added to INSTALLED_APPS."
        )


class MFASetupView(APIView):
    permission_classes = [IsAuthenticatedOrHasMFASetupChallenge]

    def post(self, request: Request) -> Response:
        if get_mfa_totp_mode() == MFA_TOTP_DISABLED:
            return Response(
                {"detail": "MFA TOTP is disabled."}, status=status.HTTP_403_FORBIDDEN)

        if Authenticator is None:
            return Response(
                {"detail": "allauth.mfa is not installed."}, status=status.HTTP_501_NOT_IMPLEMENTED)

        # Determine user: JWT auth or MFA setup bootstrap
        if request.user and request.user.is_authenticated:
            user = get_user_model().objects.get(id=request.user.id)
        elif hasattr(request, "mfa_setup_user"):
            user = request.mfa_setup_user
        else:
            return Response(
                {"detail": "Authentication credentials were not provided."},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        if Authenticator.objects.filter(user_id=user.id, type=Authenticator.Type.TOTP.value).exists():
            return Response({"detail": "TOTP already activated."}, status=status.HTTP_400_BAD_REQUEST)

        # Generate TOTP secret using django-allauth's native function
        secret = generate_totp_secret()

        # Store secret in cache with TTL
        cache_key = f"mfa_setup:{user.id}"
        cache.set(cache_key, secret, timeout=MFA_TOKEN_MAX_AGE_SECONDS)

        # Build provisioning URI and QR code using django-allauth's adapter
        adapter = get_adapter()
        provisioning_uri = adapter.build_totp_url(user, secret)
        totp_svg = adapter.build_totp_svg(provisioning_uri)

        return Response({
            "secret": secret,
            "provisioning_uri": provisioning_uri,
            "qr_code": totp_svg,
        })


class MFAActivateView(APIView):
    permission_classes = [IsAuthenticatedOrHasMFASetupChallenge]
    serializer_class = MFAActivateSerializer

    def post(self, request: Request) -> Response:
        if get_mfa_totp_mode() == MFA_TOTP_DISABLED:
            return Response(
                {"detail": "MFA TOTP is disabled."}, status=status.HTTP_403_FORBIDDEN)

        if Authenticator is None or RecoveryCodes is None:
            return Response(
                {"detail": "allauth.mfa is not installed."}, status=status.HTTP_501_NOT_IMPLEMENTED)

        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        # Determine user: JWT auth or MFA setup bootstrap
        if request.user and request.user.is_authenticated:
            user = get_user_model().objects.get(id=request.user.id)
        elif hasattr(request, "mfa_setup_user"):
            user = request.mfa_setup_user
        else:
            return Response(
                {"detail": "Authentication credentials were not provided."},
                status=status.HTTP_401_UNAUTHORIZED,
            )

        # Retrieve secret from cache
        cache_key = f"mfa_setup:{user.id}"
        secret = cache.get(cache_key)
        if not secret:
            return Response({"detail": "Setup not initiated."}, status=status.HTTP_400_BAD_REQUEST)

        code = serializer.validated_data["code"]

        # Create temporary TOTP instance to validate the code
        temp_totp = TOTP.activate(user, secret)
        if not temp_totp.validate_code(code):
            # Delete the authenticator if validation fails
            temp_totp.instance.delete()
            return Response({"detail": "Invalid code."}, status=status.HTTP_400_BAD_REQUEST)

        # Delete secret from cache after successful verification
        cache.delete(cache_key)

        recovery = RecoveryCodes.activate(user)
        recovery_codes = recovery.get_unused_codes()

        # Clean up setup_challenge if provided
        setup_challenge_id = serializer.validated_data.get("setup_challenge_id")
        is_bootstrap = bool(setup_challenge_id)
        if setup_challenge_id:
            cache.delete(f"mfa_setup_challenge:{setup_challenge_id}")

        # If this is a bootstrap flow in REQUIRED mode (setup_challenge_id present),
        # issue tokens for immediate login/registration completion.
        # This covers both login bootstrap and registration bootstrap flows.
        if is_bootstrap and get_mfa_totp_mode() == MFA_TOTP_REQUIRED:
            refresh = RefreshToken.for_user(user)
            # Use build_token_response to respect cookie configuration
            return build_token_response(
                refresh_token=refresh,
                extra_data={"success": True, "recovery_codes": recovery_codes},
                http_status=status.HTTP_200_OK
            )

        # Normal mode: just return success and recovery codes
        return Response({"success": True, "recovery_codes": recovery_codes})


class MFAListAuthenticatorsView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request: Request) -> Response:
        if get_mfa_totp_mode() == MFA_TOTP_DISABLED:
            return Response(
                {"detail": "MFA TOTP is disabled."}, status=status.HTTP_403_FORBIDDEN)

        if Authenticator is None:
            return Response({"detail": "allauth.mfa is not installed."}, status=status.HTTP_501_NOT_IMPLEMENTED)

        authenticators = Authenticator.objects.filter(user_id=request.user.id).order_by("id")
        serializer = AuthenticatorSerializer(authenticators, many=True)
        return Response(serializer.data)


class MFADeactivateView(APIView):
    permission_classes = [IsAuthenticated]
    serializer_class = MFADeactivateSerializer

    @load_user
    def post(self, request: Request) -> Response:
        if get_mfa_totp_mode() == MFA_TOTP_DISABLED:
            return Response(
                {"detail": "MFA TOTP is disabled."}, status=status.HTTP_403_FORBIDDEN)
        if get_mfa_totp_mode() == MFA_TOTP_REQUIRED:
            return Response(
                {"detail": "MFA TOTP is required and cannot be disabled."}, status=status.HTTP_403_FORBIDDEN)

        if Authenticator is None:
            return Response({"detail": "allauth.mfa is not installed."}, status=status.HTTP_501_NOT_IMPLEMENTED)

        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        if not request.user.check_password(serializer.validated_data["password"]):
            return Response({"detail": "Invalid password."}, status=status.HTTP_400_BAD_REQUEST)

        # Delete both TOTP and recovery code authenticators for the user
        deleted, _ = Authenticator.objects.filter(
            user_id=request.user.id,
            type__in=[
                Authenticator.Type.TOTP.value,
                getattr(Authenticator.Type, "RECOVERY_CODES", Authenticator.Type.RECOVERY_CODES).value
                if hasattr(Authenticator.Type, "RECOVERY_CODES") and hasattr(Authenticator.Type.RECOVERY_CODES, "value")
                else getattr(Authenticator.Type, "RECOVERY_CODES", Authenticator.Type.RECOVERY_CODES)
            ],
        ).delete()
        if deleted == 0:
            return Response({"detail": "TOTP not activated."}, status=status.HTTP_400_BAD_REQUEST)
        return Response({"success": True})


class MFAVerifyView(APIView):
    serializer_class = MFAVerifySerializer

    def post(self, request: Request) -> Response:
        if get_mfa_totp_mode() == MFA_TOTP_DISABLED:
            return Response(
                {"detail": "MFA TOTP is disabled."}, status=status.HTTP_403_FORBIDDEN)
        if Authenticator is None:
            return Response({"detail": "allauth.mfa is not installed."}, status=status.HTTP_501_NOT_IMPLEMENTED)

        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        challenge_id = serializer.validated_data["challenge_id"]
        code = serializer.validated_data["code"]

        # Retrieve challenge from cache
        challenge_data = cache.get(f"mfa_challenge:{challenge_id}")
        if not challenge_data:
            return Response({"detail": "Challenge expired or invalid."}, status=status.HTTP_400_BAD_REQUEST)

        user_id = challenge_data.get("user_id")
        User = get_user_model()
        try:
            user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            return Response({"detail": "User not found."}, status=status.HTTP_404_NOT_FOUND)

        auth_qs = Authenticator.objects.filter(user_id=user.id, type=Authenticator.Type.TOTP.value)
        if not auth_qs.exists():
            return Response({"detail": "TOTP not activated."}, status=status.HTTP_400_BAD_REQUEST)
        authenticator = auth_qs.first()

        # Validate TOTP code using django-allauth's TOTP class
        totp = TOTP(authenticator)
        if not totp.validate_code(code):
            return Response({"detail": "Invalid code."}, status=status.HTTP_400_BAD_REQUEST)

        # Delete challenge after successful verification
        cache.delete(f"mfa_challenge:{challenge_id}")

        refresh = RefreshToken.for_user(user)
        return build_token_response(refresh)


class MFAVerifyRecoveryView(APIView):
    serializer_class = MFAVerifyRecoverySerializer

    def post(self, request: Request) -> Response:
        if get_mfa_totp_mode() == MFA_TOTP_DISABLED:
            return Response(
                {"detail": "MFA TOTP is disabled."}, status=status.HTTP_403_FORBIDDEN)
        if Authenticator is None or RecoveryCodes is None:
            return Response({"detail": "allauth.mfa is not installed."}, status=status.HTTP_501_NOT_IMPLEMENTED)

        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)

        challenge_id = serializer.validated_data["challenge_id"]
        recovery_code = serializer.validated_data["recovery_code"]

        # Retrieve challenge from cache
        challenge_data = cache.get(f"mfa_challenge:{challenge_id}")
        if not challenge_data:
            return Response({"detail": "Challenge expired or invalid."}, status=status.HTTP_400_BAD_REQUEST)

        user_id = challenge_data.get("user_id")
        User = get_user_model()
        try:
            user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            return Response({"detail": "User not found."}, status=status.HTTP_404_NOT_FOUND)

        # Get recovery codes authenticator for the user
        rc_authenticator = Authenticator.objects.filter(
            user_id=user.id, type=Authenticator.Type.RECOVERY_CODES.value
        ).first()
        if not rc_authenticator:
            return Response({"detail": "Recovery codes not available."}, status=status.HTTP_400_BAD_REQUEST)

        # Validate recovery code using django-allauth's RecoveryCodes class
        rc = RecoveryCodes(rc_authenticator)
        if not rc.validate_code(recovery_code):
            return Response({"detail": "Invalid recovery code."}, status=status.HTTP_400_BAD_REQUEST)

        # Delete challenge after successful verification
        cache.delete(f"mfa_challenge:{challenge_id}")

        refresh = RefreshToken.for_user(user)
        return build_token_response(refresh)



from django.urls import path

from .views import (
    MFASetupView,
    MFAActivateView,
    MFAVerifyView,
    MFAVerifyRecoveryView,
    MFADeactivateView,
    MFAListAuthenticatorsView,
)

urlpatterns = [
    path("setup/", MFASetupView.as_view(), name="jwt_allauth_mfa_setup"),
    path("activate/", MFAActivateView.as_view(), name="jwt_allauth_mfa_activate"),
    path("verify/", MFAVerifyView.as_view(), name="jwt_allauth_mfa_verify"),
    path("verify-recovery/", MFAVerifyRecoveryView.as_view(), name="jwt_allauth_mfa_verify_recovery"),
    path("deactivate/", MFADeactivateView.as_view(), name="jwt_allauth_mfa_deactivate"),
    path("authenticators/", MFAListAuthenticatorsView.as_view(), name="jwt_allauth_mfa_authenticators"),
]



"""
Permissions for MFA endpoints.
"""
from django.contrib.auth import get_user_model
from django.core.cache import cache
from rest_framework import permissions


class IsAuthenticatedOrHasMFASetupChallenge(permissions.BasePermission):
    """
    Allows access if:
    - The user is authenticated via JWT (IsAuthenticated), or
    - The request provides a valid setup_challenge_id for MFA bootstrap.

    In the bootstrap case (setup_challenge_id), attaches request.mfa_setup_user
    with the user object loaded from the challenge data.

    This permission enables users to access /mfa/setup/ and /mfa/activate/
    endpoints during the MFA bootstrap process without a full JWT session token.
    The setup_challenge_id is issued by /login/ when MFA is REQUIRED but not yet configured.
    """

    def has_permission(self, request, view):
        # Case 1: Normal JWT authentication
        if request.user and request.user.is_authenticated:
            return True

        # Case 2: Bootstrap MFA via setup_challenge_id
        setup_challenge_id = request.data.get("setup_challenge_id")
        if not setup_challenge_id:
            return False

        data = cache.get(f"mfa_setup_challenge:{setup_challenge_id}")
        if not data:
            return False

        user_id = data.get("user_id")
        if not user_id:
            return False

        User = get_user_model()
        try:
            request.mfa_setup_user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            return False

        return True



from typing import Dict, Any
import uuid

from django.conf import settings
from django.contrib.auth.models import update_last_login
from django.core.cache import cache
from django.db import transaction
from rest_framework import exceptions
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer
from rest_framework_simplejwt.settings import api_settings

from jwt_allauth.constants import (
    MFA_TOKEN_MAX_AGE_SECONDS,
    MFA_TOTP_DISABLED,
    MFA_TOTP_REQUIRED,
)
from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.utils import allauth_authenticate


def get_mfa_totp_mode() -> str:
    """
    Return the current MFA TOTP mode from settings.

    This must be evaluated at call time (not import time) so that
    Django's `override_settings` used in tests – and any runtime changes
    – are respected.
    """
    return getattr(settings, "JWT_ALLAUTH_MFA_TOTP_MODE", MFA_TOTP_DISABLED)

try:
    from allauth.mfa.models import Authenticator  # type: ignore
except Exception:  # pragma: no cover - optional dependency guard
    Authenticator = None  # type: ignore
    if get_mfa_totp_mode() != MFA_TOTP_DISABLED:
        raise Exception(
            "MFA TOTP is not available. Please ensure 'django-jwt-allauth[mfa]' "
            "is installed and 'allauth.mfa' is added to INSTALLED_APPS."
        )


class LoginSerializer(TokenObtainPairSerializer):
    token_class = RefreshToken
    username_field = getattr(settings, 'ACCOUNT_AUTHENTICATION_METHOD', 'email')
    user = None

    @classmethod
    def get_token(cls, user) -> RefreshToken:
        """
        Instantiates a new TokenObtainPairSerializer object, sets a token for the given user and returns the token.
        """
        cls.token = cls.token_class.for_user(user)
        return cls.token  # type: ignore

    @transaction.atomic
    def validate(self, attrs: Dict[str, Any]) -> Dict[Any, Any]:
        # Get the email and password information
        authenticate_kwargs = {
            self.username_field: attrs[self.username_field],
            "password": attrs["password"],
        }
        try:
            authenticate_kwargs["request"] = self.context["request"]
        except KeyError:
            pass

        # User authentication (allauth)
        self.user = allauth_authenticate(**authenticate_kwargs)

        # Active account check
        if not api_settings.USER_AUTHENTICATION_RULE(self.user):
            raise exceptions.AuthenticationFailed(
                self.error_messages["no_active_account"],
                "no_active_account",
            )

        # MFA TOTP check
        mfa_mode = get_mfa_totp_mode()
        if mfa_mode != MFA_TOTP_DISABLED and Authenticator is not None:
            has_mfa = Authenticator.objects.filter(
                user=self.user,
                type=getattr(Authenticator, "Type").TOTP if hasattr(Authenticator, "Type") else "totp",
            ).exists()

            # If MFA is REQUIRED, user must have MFA enabled
            # Instead of raising 403, return setup challenge for bootstrap
            if mfa_mode == MFA_TOTP_REQUIRED and not has_mfa:
                setup_challenge_id = str(uuid.uuid4())
                cache.set(
                    f"mfa_setup_challenge:{setup_challenge_id}",
                    {"user_id": self.user.id},
                    timeout=MFA_TOKEN_MAX_AGE_SECONDS,
                )
                return {
                    "mfa_setup_required": True,
                    "setup_challenge_id": setup_challenge_id,
                }

            # If user has MFA enabled (OPTIONAL or REQUIRED mode), request MFA verification
            if has_mfa:
                # Store MFA challenge server-side in cache with TTL
                challenge_id = str(uuid.uuid4())
                challenge_data = {"user_id": self.user.id, "timestamp": None}
                cache.set(f"mfa_challenge:{challenge_id}", challenge_data, timeout=MFA_TOKEN_MAX_AGE_SECONDS)
                return {"mfa_required": True, "challenge_id": challenge_id}

        validated_data = super().validate(attrs)

        # Set the refresh token
        refresh = self.get_token(self.user)

        validated_data["refresh"] = str(refresh)
        validated_data["access"] = str(refresh.access_token)

        if api_settings.UPDATE_LAST_LOGIN:
            update_last_login(None, self.user)

        return validated_data



PASS_RESET = 'PASS_RESET'
PASS_RESET_ACCESS = 'PASS_RESET_ACCESS'
TEMPLATE_PATHS = 'JWT_ALLAUTH_TEMPLATES'

EMAIL_VERIFIED_REDIRECT = 'EMAIL_VERIFIED_REDIRECT'
PASSWORD_RESET_REDIRECT = 'PASSWORD_RESET_REDIRECT'

PASS_RESET_COOKIE = 'password_reset_access_token'

FOR_USER = 'for_user'
ONE_TIME_PERMISSION = 'one_time_permission'

REFRESH_TOKEN_COOKIE = 'refresh_token'

# Admin-managed registration & email confirmation flow
PASS_SET = 'PASS_SET'
PASS_SET_ACCESS = 'PASS_SET_ACCESS'
PASSWORD_SET_REDIRECT = 'PASSWORD_SET_REDIRECT'
SET_PASSWORD_COOKIE = 'set_password_access_token'
EMAIL_CONFIRMATION = 'EMAIL_CONFIRMATION'

# MFA
MFA_SALT = 'jwt_allauth_mfa'
MFA_TOKEN_MAX_AGE_SECONDS = 300

# MFA TOTP modes
MFA_TOTP_DISABLED = 'disabled'
MFA_TOTP_OPTIONAL = 'optional'
MFA_TOTP_REQUIRED = 'required'



from allauth.account.views import ConfirmEmailView
from django.conf import settings
from django.http import HttpResponseNotAllowed, HttpResponseRedirect
from django.urls import reverse
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView
from rest_framework_simplejwt.exceptions import InvalidToken

from jwt_allauth.constants import (
    EMAIL_VERIFIED_REDIRECT,
    PASSWORD_SET_REDIRECT,
    FOR_USER,
    ONE_TIME_PERMISSION,
    PASS_SET_ACCESS,
    SET_PASSWORD_COOKIE,
    EMAIL_CONFIRMATION,
)
from jwt_allauth.registration.email_verification.serializers import VerifyEmailSerializer
from jwt_allauth.tokens.app_settings import RefreshToken
from jwt_allauth.tokens.models import GenericTokenModel, RefreshTokenWhitelistModel
from jwt_allauth.tokens.serializers import GenericTokenModelSerializer


class VerifyEmailView(APIView, ConfirmEmailView):
    permission_classes = (AllowAny,)
    allowed_methods = ('GET',)
    # URL where the frontend password-set flow is implemented (admin-managed registration)
    # By default, point to the built-in HTML UI provided by this library.
    form_url = getattr(settings, PASSWORD_SET_REDIRECT, '/registration/set-password/default/')

    @staticmethod
    def get_serializer(*args, **kwargs):
        return VerifyEmailSerializer(*args, **kwargs)

    def get(self, request, *args, **kwargs):
        # If admin-managed registration is enabled, validate the confirmation key and
        # issue a one-time token to allow the user to set their password.
        if getattr(settings, 'JWT_ALLAUTH_ADMIN_MANAGED_REGISTRATION', False):
            # Ensure PASSWORD_SET_REDIRECT has been configured
            if self.form_url is None:
                raise NotImplementedError('`PASSWORD_SET_REDIRECT` must be configured in settings.py')

            # Check that the email confirmation token has not been used already
            query_set = GenericTokenModel.objects.filter(
                token=kwargs['key'],
                purpose=EMAIL_CONFIRMATION,
            )
            if len(query_set) != 1:
                raise InvalidToken()
            # Single-use confirmation token
            query_set.delete()

            confirmation = self.get_object()
            user = confirmation.email_address.user
            # Confirm the email now
            confirmation.confirm(self.request)

            # Create one-time access token to allow setting the password
            refresh_token = RefreshToken()
            refresh_token[FOR_USER] = user.id
            refresh_token[ONE_TIME_PERMISSION] = PASS_SET_ACCESS
            access_token = refresh_token.access_token

            response = HttpResponseRedirect(self.form_url)
            response.set_cookie(
                key=SET_PASSWORD_COOKIE,
                value=str(access_token),
                httponly=getattr(settings, 'PASSWORD_SET_COOKIE_HTTP_ONLY', True),
                secure=getattr(settings, 'PASSWORD_SET_COOKIE_SECURE', not settings.DEBUG),
                samesite=getattr(settings, 'PASSWORD_SET_COOKIE_SAME_SITE', 'Lax'),
                max_age=getattr(settings, 'PASSWORD_SET_COOKIE_MAX_AGE', 3600 * 24),
            )

            token_serializer = GenericTokenModelSerializer(
                data={
                    'token': access_token['jti'],
                    'user': user.id,
                    'purpose': PASS_SET_ACCESS,
                }
            )
            token_serializer.is_valid(raise_exception=True)
            token_serializer.save()

            return response

        # Default flow: just confirm the email and enable refresh tokens
        confirmation = self.get_object()

        # Enable refresh token
        refresh = RefreshTokenWhitelistModel.objects.filter(user=confirmation.email_address.user).first()
        if refresh:
            refresh.enabled = True
            refresh.save()

        confirmation.confirm(self.request)
        return HttpResponseRedirect(
            getattr(settings, EMAIL_VERIFIED_REDIRECT, reverse('jwt_allauth_email_verified'))
        )

    def post(self, request, *args, **kwargs):
        return HttpResponseNotAllowed(['GET'])



from rest_framework.permissions import BasePermission as DefaultBasePermission
from rest_framework_simplejwt.exceptions import TokenError

from jwt_allauth.constants import PASS_RESET_ACCESS, PASS_RESET_COOKIE, FOR_USER, ONE_TIME_PERMISSION, \
    PASS_SET_ACCESS, SET_PASSWORD_COOKIE
from jwt_allauth.password_reset.models import SetPasswordTokenUser
from jwt_allauth.tokens.app_settings import RefreshToken


class _BaseOneTimeCookiePermission(DefaultBasePermission):
    """
    Base permission that validates a one-time access token from a specific cookie
    and injects a SetPasswordTokenUser into the request.
    Subclasses must define COOKIE_NAME and REQUIRED_PERMISSION.
    """
    COOKIE_NAME = None
    REQUIRED_PERMISSION = None

    def has_permission(self, request, view):
        if bool(request.user and request.user.is_authenticated):
            return False
        if not self.COOKIE_NAME or not self.REQUIRED_PERMISSION:
            return False
        if hasattr(request, 'COOKIES') and self.COOKIE_NAME in request.COOKIES:
            access_token = request.COOKIES.get(self.COOKIE_NAME)
            try:
                access_token = RefreshToken.access_token_class(access_token)
            except TokenError:
                return False
            if access_token and ONE_TIME_PERMISSION in access_token and FOR_USER in access_token:
                if access_token[ONE_TIME_PERMISSION] == self.REQUIRED_PERMISSION:
                    request.user = SetPasswordTokenUser(access_token)
                    request.auth = access_token
                    return True
        return False


class ResetPasswordPermission(_BaseOneTimeCookiePermission):
    COOKIE_NAME = PASS_RESET_COOKIE
    REQUIRED_PERMISSION = PASS_RESET_ACCESS


class SetPasswordPermission(_BaseOneTimeCookiePermission):
    COOKIE_NAME = SET_PASSWORD_COOKIE
    REQUIRED_PERMISSION = PASS_SET_ACCESS



from rest_framework import serializers


try:
    from allauth.mfa.models import Authenticator as AuthModel  # type: ignore
except Exception:  # pragma: no cover - optional dependency guard
    AuthModel = None  # type: ignore


class MFAActivateSerializer(serializers.Serializer):
    """
    Activates TOTP for the authenticated user by providing a 6-digit code.

    Can be used in two ways:
    - With JWT authentication (normal case)
    - With setup_challenge_id (bootstrap MFA after login when REQUIRED mode)
    """
    code = serializers.CharField(min_length=6, max_length=6)
    setup_challenge_id = serializers.CharField(required=False, allow_blank=True)


class MFAVerifySerializer(serializers.Serializer):
    """
    Verifies a TOTP code during login using a server-side challenge.
    """
    challenge_id = serializers.CharField()
    code = serializers.CharField(min_length=6, max_length=6)


class MFAVerifyRecoverySerializer(serializers.Serializer):
    """
    Verifies a recovery code during login using a server-side challenge.
    """
    challenge_id = serializers.CharField()
    recovery_code = serializers.CharField()


class MFADeactivateSerializer(serializers.Serializer):
    """
    Deactivates TOTP for the authenticated user.
    """
    password = serializers.CharField(write_only=True)


if AuthModel is not None:
    class AuthenticatorSerializer(serializers.ModelSerializer):  # type: ignore[misc]
        class Meta:
            model = AuthModel
            fields = ("id", "type", "created_at", "last_used_at")
            read_only_fields = fields
else:
    class AuthenticatorSerializer(serializers.Serializer):
        """
        Fallback serializer when allauth.mfa is not available.
        """
        id = serializers.IntegerField()
        type = serializers.CharField()
        created_at = serializers.DateTimeField()
        last_used_at = serializers.DateTimeField(allow_null=True)
